from django.conf import settings
from django.conf import LazyObject

from django.utils.translation import ugettext_lazy as _

import os


class OscarWebpaySettingError(Exception):
    pass


class OscarWebpaySettings(object):

    def __init__(self):

        self.WEBPAY_RETURN_IP_ADDRESS = getattr(settings, 'WEBPAY_RETURN_IP_ADDRESS', '127.0.0.1')
        self.WEBPAY_RETURN_PORT = getattr(settings, 'WEBPAY_RETURN_PORT', 80)
        self.WEBPAY_CERT_DIR = getattr(settings, 'WEBPAY_CERT_DIR', os.path.join(settings.BASE_DIR, 'webpay_certificates'))
        self.WEBPAY_ENABLED_MODES = getattr(settings, 'WEBPAY_ENABLED_MODES', ['NORMAL'])
        self.__webpay_modes = ['NORMAL', 'NORMAL_MALL', 'CAPTURE', 'ONECLICK', 'COMPLETE', 'NULLIFY']
        self.__conf_required_data = [
            'ACTIVE_ENVIRON',   # Use mode (INTEGRATION, CERTIFICATION o PRODUCTION).
            'ENVIRONMENTS',     # Dictionary, keys are environment and values are the WSDL values.
            'PRIVATE_KEY',      # Private key, absolute path.
            'PUBLIC_CERT',      # Public certificate, absolute path.
            'WEBPAY_CERT',      # Private cert, absolute path.
            'COMMERCE_CODE',    # Well ...   Commerce code ;)
        ]

        for mode in self.WEBPAY_ENABLED_MODES:
            if mode in self.__webpay_modes:
                webpay_mode = getattr(settings, 'WEBPAY_{}'.format(mode), None)
                if webpay_mode is None:
                    raise OscarWebpaySettingError(
                        _(u'You must set the configuration for all the enabled webpay modes\n'
                          u'Please configure WEBPAY_{} setting\'s variable.'.format(mode))
                    )
                else:
                    setattr(self, 'WEBPAY_{}'.format(mode), webpay_mode)

                # Check all the required data is set.
                for key in self.__conf_required_data:
                    if key not in webpay_mode.keys():
                        raise OscarWebpaySettingError(_(u'WEBPAY_{} "{}" setting has not been set.').format(mode, key))
            else:
                raise OscarWebpaySettingError(_(u'Oscar webpay do not support this mode: {}'.format(mode)))


class OscarWebpayLazySettings(LazyObject):

    def _setup(self, name=None):
        self._wrapped = OscarWebpaySettings()


oscar_webpay_settings = OscarWebpayLazySettings()



from oscar_webpay.libwebpay.webpay import Webpay
from oscar_webpay.certificates import cert_normal
from oscar_webpay.libwebpay.configuration import Configuration

from oscar_webpay.oscar_webpay_settings import oscar_webpay_settings
from django.core.urlresolvers import reverse


def get_webpay_conf():
    certificate = cert_normal.certDictionary.dictionaryCert()
    configuration = Configuration()
    configuration.setEnvironment(certificate['environment'])
    configuration.setCommerceCode(certificate['commerce_code'])
    configuration.setPrivateKey(certificate['private_key'])
    configuration.setPublicCert(certificate['public_cert'])
    configuration.setWebpayCert(certificate['webpay_cert'])
    return configuration


def get_webpay_client(order_number, total):

    webpay = Webpay(get_webpay_conf())

    amount = total
    buy_order = order_number
    session_id = None

    final_url = 'http://{}:{}{}'.format(
        oscar_webpay_settings.WEBPAY_RETURN_IP_ADDRESS,
        oscar_webpay_settings.WEBPAY_RETURN_PORT,
        reverse('webpay-cancel')
    )

    return_url = 'http://{}:{}{}'.format(
        oscar_webpay_settings.WEBPAY_RETURN_IP_ADDRESS,
        oscar_webpay_settings.WEBPAY_RETURN_PORT,
        reverse('webpay-details-returns')
    )

    client = webpay.getNormalTransaction().initTransaction(amount, buy_order, session_id, return_url, final_url)
    return client


def confirm_transaction(token):
    webpay = Webpay(get_webpay_conf())
    return webpay.getNormalTransaction().getTransaction(token)





import os
from django.conf import settings

OSCAR_WEBPAY_TEMPLATES_DIR = os.path.join(
    os.path.dirname(__file__), 'templates'
)


settings.LOCALE_PATHS = settings.LOCALE_PATHS + (os.path.join(os.path.dirname(__file__), 'locale/'), )



from django.conf.urls import patterns, url
from oscar_webpay import views


urlpatterns = patterns(
    '',
    url(r'^webpay_payment/$', views.WebPayRedirect.as_view(), name='webpay-payment'),
    url(r'^webpay_detail/$', views.WebPayPaymentDetailsView.as_view(), name='webpay-details'),
    # We use a specific template here because we inject the
    # WebPay url for the transaction to the form action attribute.
    url(r'^webpay_detail/returns/$', views.WebPayPaymentDetailsView.as_view(
        payment_mode=True
    ), name='webpay-details-returns'),
    url(r'^webpay_fail/$', views.WebPayFail.as_view(), name='webpay-fail'),
    url(r'^webpay_form/$', views.WebPayForm.as_view(), name='webpay-form'),
    url(r'^webpay_cancel/$', views.WebPayCancel.as_view(), name='webpay-cancel'),
    url(r'^webpay_thanks/$', views.WebPayThankYouView.as_view(), name='webpay-tnx'),
    url(r'^webpay_thanks/$', views.WebPaySuccessView.as_view(), name='webpay-success'),
)
